from . import godot_script_1
from . import godot_script_2
from . import csharp
from . import resource
from . import json

registers = {}

registers['GDS1.0'] = godot_script_1
registers['GDS2.0'] = godot_script_2
registers['C#'] = csharp
registers['RESOURCE'] = resource
registers['JSON1.0'] = json
registers['JSON2.0'] = json
